/**
 * 
 */
package facture.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;

import org.hsqldb.jdbc.jdbcStatement;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcDaoSupport;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.jdbc.core.support.JdbcDaoSupport;

import facture.modele.Article;

/**
 *
 */
public class ArticleDAOJdbc extends NamedParameterJdbcDaoSupport implements ArticleDAO {

	/* (non-Javadoc)
	 * @see facture.dao.ArticleDAO#createArticle(facture.modele.Article)
	 */
	public void createArticle(Article a) {
		// Comme on veut sauver tous les champs d'un article, on va 
		// passer par un bean article et des paramètres nommés.
		
		String sql = "insert into Article values (:id, :designation, :prix)";

	    SqlParameterSource param = new BeanPropertySqlParameterSource(a);

	    getNamedParameterJdbcTemplate().update(sql, param);		
	}

	/* (non-Javadoc)
	 * @see facture.dao.ArticleDAO#deleteArticle(int)
	 */
	public void deleteArticle(int id) {
		String sql= "delete from article where idArticle= :id";
		HashMap<String, Object> map= new HashMap<String, Object>();
		map.put("id", new Integer(id));
		getNamedParameterJdbcTemplate().update(sql, map);
		//getJdbcTemplate().update(sql, new Object[] {Integer.valueOf(id)});
		//getJdbcTemplate().queryForInt(sql)
	}

	/* (non-Javadoc)
	 * @see facture.dao.ArticleDAO#findArticleById(int)
	 */
	public Article findArticleById(int id) {
		String sql= "select * from Article where idArticle= :id";
		SimpleJdbcTemplate simpleJdbcTemplate= new SimpleJdbcTemplate(getDataSource());

		return simpleJdbcTemplate.queryForObject(sql,new ArticleMapper(),id);
		
	}

	public List<Article> findArticlesMoinsCherQue(double prix) {
		String sql= "select * from Article where prix < ?";
		SimpleJdbcTemplate simpleJdbcTemplate= new SimpleJdbcTemplate(getDataSource());
		return simpleJdbcTemplate.query(sql, new ArticleMapper(), prix);
	}
	
	/* (non-Javadoc)
	 * @see facture.dao.ArticleDAO#updateArticle(facture.modele.Article)
	 */
	public void updateArticle(Article a) {
		String sql= "update article set designation = :designation, prix= :prix where idArticle= :id";

		SqlParameterSource param = new BeanPropertySqlParameterSource(a);

	    getNamedParameterJdbcTemplate().update(sql, param);		
		
	}

	private class ArticleMapper implements ParameterizedRowMapper<Article> {

		/* (non-Javadoc)
		 * @see org.springframework.jdbc.core.RowMapper#mapRow(java.sql.ResultSet, int)
		 */
		public Article mapRow(ResultSet r, int rowNum) throws SQLException {
			Article a= new Article();
			a.setId(r.getLong("idArticle"));
			a.setDesignation(r.getString("description"));
			a.setPrix(r.getDouble("prix"));
			return a;
		}
	}
	
	public long getNextId() {
		String sql= "select max(idArticle) from Article";
		return getJdbcTemplate().queryForLong(sql) + 1;
	}
}
